"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodePoolWorker = void 0;
const node_util_1 = require("node:util");
const microservice_1 = require("@jfrog-pipelines/microservice");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const modules_1 = require("./modules");
const ORPHAN_NODES_COLLECTION_INTERVAL_IN_MS = 30 * 60 * 1000;
class NodePoolWorker {
    constructor(details, store) {
        this.details = details;
        this.store = store;
        this.logger = logger_1.LoggerFactory.getLogger('nodePoolWorker');
        this.nodePoolService = new modules_1.NodePoolService();
        this.systemProperties = configuration_1.ConfigFactory.config;
    }
    async process(message) {
        const publisher = new amqp_client_1.AmqpPublisher(amqp_client_1.AmqpFactory.getClient(), this.systemProperties.amqpExchange, amqp_client_1.publisherOptions, this.logger);
        const payload = message?.payload;
        if (payload) {
            const poolID = payload.nodePoolId;
            const systemInstanceId = this.store.get(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER).systemServiceInstancesId;
            this.logger.trace(`Processing for node pool with id: ${poolID} with system service id: ${systemInstanceId}`);
            const lockService = new modules_1.LockService(systemInstanceId);
            const lockName = [...this.details.name, '-nodePools-', ...String(poolID)].join('');
            try {
                this.logger.trace(`Fetching node pool with id: ${poolID}`);
                const nodePool = await this.nodePoolService.getNodePool(poolID);
                if (!nodePool) {
                    this.logger.info(`No valid node pool found for id ${poolID}`);
                    return;
                }
                this.logger.trace(`Succesfully fetched node pool with id: ${poolID} and node pool name is ${nodePool.name}`);
                if (!nodePool.isOnDemand) {
                    this.logger.debug(`Node Pool is not on demand, wont continue with execution`);
                    return;
                }
                const collector = new modules_1.NodeCollector(publisher);
                const provisioner = new modules_1.NodeProvisioner(publisher);
                this.logger.trace(`Trying to obtain lock for node pool ${nodePool.name} with lock name: ${lockName}`);
                const lock = await lockService.obtainLock({
                    name: lockName,
                    lockTimeoutInMs: modules_1.LockTimeoutInMs
                });
                if (!lock) {
                    this.logger.warn(`Failed to obtain lock for node pool id ${poolID}`);
                    return;
                }
                this.logger.trace(`Succesfully obtained lock for node pool ${nodePool.name} with lock name: ${lockName}`);
                await provisioner.provision(nodePool);
                await collector.deProvision(nodePool);
            }
            catch (error) {
                this.logger.error(`Failed to process node pool id ${poolID} with err ${(0, node_util_1.inspect)(error)}`);
            }
            finally {
                this.logger.trace(`Trying to release lock  name: ${lockName}`);
                await lockService.releaseLock(lockName);
                this.logger.trace(`Lock was released succesfully with name: ${lockName}`);
            }
        }
    }
    async startCollectingOrphanNodes() {
        this.logger.info('Started collection of Orphan nodes');
        const publisher = new amqp_client_1.AmqpPublisher(amqp_client_1.AmqpFactory.getClient(), this.systemProperties.amqpExchange, amqp_client_1.publisherOptions, this.logger);
        const collector = new modules_1.NodeCollector(publisher);
        return this.collectOrphanNodes(collector);
    }
    async collectOrphanNodes(collector) {
        this.logger.debug('Collecting orphan nodes started');
        await collector.collectOrphanNodes();
        this.logger.debug(`Orphan nodes were succesfully collected. Will be recollected in next ${ORPHAN_NODES_COLLECTION_INTERVAL_IN_MS / 1000 / 60} mins`);
        setTimeout(() => {
            this.collectOrphanNodes(collector);
        }, ORPHAN_NODES_COLLECTION_INTERVAL_IN_MS);
    }
}
exports.NodePoolWorker = NodePoolWorker;
//# sourceMappingURL=NodePoolWorker.js.map