"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_crypto_1 = require("node:crypto");
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const logger_1 = require("@jfrog-pipelines/logger");
const utility_1 = require("@jfrog-pipelines/utility");
const microservice_1 = require("@jfrog-pipelines/microservice");
const NodePoolWorker_1 = require("../NodePoolWorker");
const modules_1 = require("../modules");
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog-pipelines/configuration', () => {
    return {
        ConfigFactory: { config: { amqpExchange: '' } }
    };
});
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const client = new amqp_client_1.AmqpClient('', '', { amqpUrl: '' }, logger_1.LoggerFactory.getLogger('NodePoolWorkerTest'));
const publisher = new amqp_client_1.AmqpPublisher(client, '', amqp_client_1.publisherOptions, logger_1.LoggerFactory.getLogger('NodePoolWorkerTest'));
const store = microservice_1.globalStore;
const serviceEnum = 14;
const serviceDetails = utility_1.ServiceMeta[serviceEnum];
describe('Test NodePoolWorker #process', () => {
    it('should process message as node pool id in case of node pool is on demand', async () => {
        const nodeService = new modules_1.NodePoolService();
        const provisioner = new modules_1.NodeProvisioner(publisher);
        const collector = new modules_1.NodeCollector(publisher);
        jest.spyOn(modules_1.NodeCollector.prototype, 'deProvision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodeProvisioner.prototype, 'provision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue({ id: 1, isOnDemand: true });
        const lockResponse = { id: (0, node_crypto_1.randomUUID)() };
        jest.spyOn(modules_1.LockService.prototype, 'obtainLock').mockResolvedValue(lockResponse);
        jest.spyOn(modules_1.LockService.prototype, 'releaseLock').mockResolvedValue();
        store.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
        const poolWorker = new NodePoolWorker_1.NodePoolWorker(serviceDetails, store);
        await poolWorker.process({ payload: { nodePoolId: 1 } });
        expect(nodeService.getNodePool).toHaveBeenCalled();
        expect(provisioner.provision).toHaveBeenCalledTimes(1);
        expect(collector.deProvision).toHaveBeenCalledTimes(1);
    });
    it('should not process message as node pool id in case of node pool is not on demand', async () => {
        const nodeService = new modules_1.NodePoolService();
        const provisioner = new modules_1.NodeProvisioner(publisher);
        const collector = new modules_1.NodeCollector(publisher);
        jest.spyOn(modules_1.NodeCollector.prototype, 'deProvision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodeProvisioner.prototype, 'provision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue({ id: 1, isOnDemand: false });
        store.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
        const poolWorker = new NodePoolWorker_1.NodePoolWorker(serviceDetails, store);
        await poolWorker.process({ payload: { nodePoolId: 1 } });
        expect(nodeService.getNodePool).toHaveBeenCalled();
        expect(provisioner.provision).toHaveBeenCalledTimes(0);
        expect(collector.deProvision).toHaveBeenCalledTimes(0);
    });
    it('should not throw error in case of any errors', async () => {
        const provisioner = new modules_1.NodeProvisioner(publisher);
        const collector = new modules_1.NodeCollector(publisher);
        jest.spyOn(modules_1.NodeCollector.prototype, 'deProvision').mockRejectedValue({});
        jest.spyOn(modules_1.NodeProvisioner.prototype, 'provision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue({ id: 1, isOnDemand: false });
        store.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
        const poolWorker = new NodePoolWorker_1.NodePoolWorker(serviceDetails, store);
        await poolWorker.process({ payload: { nodePoolId: 1 } });
        expect(provisioner.provision).toHaveBeenCalledTimes(0);
        expect(collector.deProvision).toHaveBeenCalledTimes(0);
    });
});
//# sourceMappingURL=NodePoolWorker.test.js.map