"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeCollector = void 0;
const node_util_1 = require("node:util");
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const NodeService_1 = require("./node/NodeService");
const TIME = {
    MILLISECOND: 1000,
    SECOND: 60,
    MIN: 60,
    HOURS: 24
};
const NODE_TIMEOUT = 15;
class NodeCollector {
    constructor(publisher) {
        this.logger = logger_1.LoggerFactory.getLogger('NodeCollector');
        this.nodeService = new NodeService_1.NodeService(publisher);
    }
    async deProvision(nodePool) {
        this.logger = logger_1.LoggerFactory.getLogger(`NodeCollector:${nodePool.id}${nodePool.name}`);
        try {
            this.logger.trace('Fetching nodes');
            const nodes = await this.getNodes(nodePool.id);
            this.logger.trace(`Active nodes are ${nodes.length}`);
            const idleNodes = this.nodeService.filterIdleNodes(nodePool, nodes);
            this.logger.trace(`idleNodes nodes are ${idleNodes.length}`);
            const deletedNodes = await this.computeDeleteState(nodePool, idleNodes);
            this.logger.trace(`Nodes to be deleted are ${deletedNodes.length}`);
            if (nodePool.isCacheEnabled) {
                const cachedNodes = this.computeCachedState(nodePool, idleNodes, deletedNodes);
                await this.applyCachedStateToNodes(cachedNodes);
            }
            await this.applyDeleteStateToNodes(deletedNodes);
        }
        catch (error) {
            this.logger.warn(`Failed to deProvision nodes for node pool ${nodePool.id} due to err ${(0, node_util_1.inspect)(error)}`);
        }
        finally {
            this.logger.trace(`Node collection completed. Releasing the access client`);
            await this.nodeService.release();
        }
    }
    async collectOrphanNodes() {
        try {
            this.logger.trace(`Fetching orphan nodes`);
            const nodes = await this.nodeService.getOrphanNodes();
            this.logger.trace(`Fetched ${nodes.length} orphan node/s`);
            await this.applyDeleteStateToNodes(nodes);
            this.logger.info(`Collected ${nodes.length} orphan nodes.`);
            return nodes.length;
        }
        catch (error) {
            this.logger.warn(`Failed to collectOrphan nodes due to err ${(0, node_util_1.inspect)(error)}`);
        }
        return 0;
    }
    async getNodes(poolID) {
        return this.nodeService.getAllNodes(poolID);
    }
    async computeDeleteState(nodePool, nodes) {
        return nodes.filter((node) => {
            return this.getNodeDeleteCondition(nodePool, node);
        });
    }
    getNodeDeleteCondition(nodePool, node) {
        const currentDate = new Date();
        const currentDateInMS = currentDate.getTime();
        const CUTOFF_TIME = NODE_TIMEOUT * TIME.SECOND * TIME.MILLISECOND;
        const cutOffTimeForWaitingAt = currentDateInMS - CUTOFF_TIME;
        const currentDateOfMonth = currentDate.getDate();
        const nodeProvisionedAt = new Date(node.provisionedAt);
        const provisionedAt = nodeProvisionedAt.getTime();
        const statusLastUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
        const daysPassedSinceProvision = (currentDateInMS - provisionedAt) / (TIME.MIN * TIME.SECOND * TIME.MILLISECOND * TIME.HOURS);
        if (node.statusCode === NodeService_1.NodeStatus.waiting && statusLastUpdatedAt < cutOffTimeForWaitingAt) {
            return true;
        }
        if (node.stopRequested) {
            return true;
        }
        if (node.statusCode === NodeService_1.NodeStatus.stopped) {
            return true;
        }
        if (node.statusCode === NodeService_1.NodeStatus.failure) {
            return true;
        }
        if (node.statusCode === NodeService_1.NodeStatus.processing) {
            return false;
        }
        if (!nodePool.isCacheEnabled) {
            return false;
        }
        if ((0, utility_1.isEmpty)(nodePool.cacheSettingsPropertyBag)) {
            return false;
        }
        if (nodePool.cacheSettingsPropertyBag.nodeStopSetting !== `nodeStopIntervalDays`) {
            return false;
        }
        if (daysPassedSinceProvision >= nodePool.cacheSettingsPropertyBag.nodeStopIntervalDays) {
            return true;
        }
        if (nodeProvisionedAt.getDate() != currentDateOfMonth) {
            return true;
        }
        return false;
    }
    computeCachedState(nodePool, idleNodes, deletedNodes) {
        const nodes = new Map();
        for (let index = 0; index <= idleNodes.length - 1; index++) {
            nodes.set(idleNodes[index].id, idleNodes[index]);
        }
        for (let index = 0; index <= deletedNodes.length - 1; index++) {
            nodes.delete(idleNodes[index].id);
        }
        return [...nodes.values()];
    }
    async applyDeleteStateToNodes(nodes) {
        this.logger.trace(`Trying to deprovision ${nodes.length} nodes`);
        const deleteStatePromises = nodes.map((node) => {
            return this.nodeService.applyNodeToStoppedState(node);
        });
        await Promise.allSettled(deleteStatePromises);
    }
    async applyCachedStateToNodes(nodes) {
        this.logger.trace(`Trying to cache ${nodes.length} nodes`);
        const cachedStatePromises = nodes.map((node) => {
            return this.nodeService.applyNodeToCachedState(node);
        });
        await Promise.allSettled(cachedStatePromises);
    }
}
exports.NodeCollector = NodeCollector;
//# sourceMappingURL=NodeCollector.js.map