"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeProvisioner = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const uuid_1 = require("uuid");
const configuration_1 = require("@jfrog-pipelines/configuration");
const BuildPlaneService_1 = require("./buildPlane/BuildPlaneService");
const NodeService_1 = require("./node/NodeService");
class NodeProvisioner {
    constructor(publisher) {
        this.logger = logger_1.LoggerFactory.getLogger('Provisioner');
        this.nodeService = new NodeService_1.NodeService(publisher);
        this.buildPlaneService = new BuildPlaneService_1.BuildPlaneService();
    }
    async provision(nodePool) {
        this.logger = logger_1.LoggerFactory.getLogger(`NodeProvisioner:${nodePool.id}:${nodePool.name}`);
        try {
            this.logger.trace('Fetching nodes');
            const nodes = await this.getNodes(nodePool.id);
            this.logger.trace(`Succesfully fetched ${nodes.length} node/s`);
            this.logger.trace('Fetching buildPlaneMessage count');
            const messageCount = await this.getBuildMessageCount(nodePool.id);
            this.logger.trace(`Current buildPlaneMessage count is ${messageCount}`);
            const activeNodes = this.nodeService.filterActiveNodes(nodes);
            this.logger.trace(`Current activeNodes count is ${activeNodes.length}`);
            const requiredNodeCount = this.calculateRequiredNodes(activeNodes, messageCount);
            this.logger.trace(`Required nodes count is ${requiredNodeCount}`);
            if (requiredNodeCount <= 0) {
                this.logger.debug(`No new nodes are required to be provisioned`);
                return;
            }
            const cachedNodes = this.nodeService.filterCachedNodes(nodes);
            this.logger.trace(`Cached nodes count is ${cachedNodes.length}`);
            const requiredNewNodes = requiredNodeCount - cachedNodes.length;
            this.logger.trace(`Required ${requiredNewNodes} more nodes to be created`);
            await this.useCacheNodes(requiredNewNodes, cachedNodes);
            if (requiredNewNodes > 0) {
                this.logger.trace(`Provisioning ${requiredNewNodes} new nodes`);
                await this.provisionActiveNodes(requiredNewNodes, nodePool);
            }
        }
        catch (error) {
            const message = `Failed to provision nodes for node pool id ${nodePool.id} due to ${(0, node_util_1.inspect)(error)}`;
            this.logger.warn(message);
            throw new Error(message);
        }
        finally {
            this.logger.trace(`Node provisioning completed. Releasing the access client`);
            await this.nodeService.release();
        }
    }
    async getNodes(poolID) {
        return this.nodeService.getRunningNodes(poolID);
    }
    async getBuildMessageCount(poolID) {
        return this.buildPlaneService.getMessageCount(poolID);
    }
    calculateRequiredNodes(nodes, messageCount) {
        const requiredCount = messageCount - nodes.length;
        return requiredCount;
    }
    async useCacheNodes(requiredCount, nodes) {
        const effectiveNodes = requiredCount < 0 ? nodes.splice(-requiredCount) : nodes;
        this.logger.trace(`Starting ${effectiveNodes} cache nodes.`);
        const cachedNodePromises = effectiveNodes.map((node) => {
            return this.nodeService.applyNodeWaitingState(node);
        });
        await Promise.allSettled(cachedNodePromises);
    }
    async provisionActiveNodes(count, nodePool) {
        const newRequiredNodes = Array.from({ length: count }).fill(null);
        const newNodePromises = newRequiredNodes.map(() => {
            const uid = (0, uuid_1.v4)();
            const friendlyName = `node-${uid.slice(Math.max(0, uid.length - 8))}`;
            const sshPort = nodePool.providerMetadataPropertyBag
                ? nodePool.providerMetadataPropertyBag.sshPort
                : 0;
            const user = nodePool.providerMetadataPropertyBag
                ? nodePool.providerMetadataPropertyBag.sshUser
                : 'root';
            const buildPlaneVersion = configuration_1.ConfigFactory.config.buildPlaneVersion;
            const nodeRequestBody = {
                projectId: nodePool.projectId,
                nodePoolId: nodePool.id,
                friendlyName: friendlyName,
                isOnDemand: true,
                statusCode: NodeService_1.NodeStatus.waiting,
                statusLastUpdatedAt: new Date(),
                createdBy: NodeService_1.NodeAccessorID,
                updatedBy: NodeService_1.NodeAccessorID,
                isAutoInitialized: false,
                stopRequested: false,
                intendedBuildPlaneVersion: buildPlaneVersion,
                projectIntegrationId: nodePool.projectIntegrationId,
                sshPort: sshPort,
                sshUser: user,
                systemPropertyBag: {},
                initPropertyBag: nodePool.initPropertyBag
            };
            return this.nodeService.createNode(nodeRequestBody);
        });
        await Promise.allSettled(newNodePromises);
        return;
    }
}
exports.NodeProvisioner = NodeProvisioner;
//# sourceMappingURL=NodeProvisioner.js.map