"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LockService = exports.LockTimerIntervalInMs = exports.LockTimeoutInMs = void 0;
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const uuid_1 = require("uuid");
const logger_1 = require("@jfrog-pipelines/logger");
const LockDao_1 = require("./LockDao");
exports.LockTimeoutInMs = 1 * 60 * 1000;
exports.LockTimerIntervalInMs = 50 * 1000;
class LockService {
    constructor(systemInstanceID) {
        this.systemInstanceID = systemInstanceID;
        this.logger = logger_1.LoggerFactory.getLogger('LockService');
        this.lockDao = new LockDao_1.LockDao();
    }
    async obtainLock(attribute) {
        try {
            const status = await this.isAlreadyLocked(attribute.name);
            if (status) {
                throw new Error(`Lock already exists with name ${attribute.name}`);
            }
        }
        catch (error) {
            this.logger.error(`Failed with retrieve existing lock with error ${error}`);
            throw error;
        }
        try {
            const lockAttributes = {
                id: (0, uuid_1.v4)(),
                name: attribute.name,
                systemServiceInstanceId: this.systemInstanceID,
                timeoutAt: new Date(Date.now() + exports.LockTimeoutInMs),
                createdAt: new Date(),
                updatedAt: new Date()
            };
            const lock = await this.lockDao.create(lockAttributes);
            this.setLockTimeoutAtInterval(lock.id, exports.LockTimeoutInMs);
            return lock;
        }
        catch (error) {
            const message = `Failed to create lock for ${attribute.name} due to err ${error.message} `;
            this.logger.warn(message);
            throw new Error(message);
        }
    }
    async releaseLock(lockName) {
        try {
            const lock = await this.findByName(lockName);
            if (!lock) {
                this.logger.warn(`No existing lock with name ${lockName}`);
                return;
            }
            await this.delete(lock.id);
        }
        catch (error) {
            this.logger.warn(`Failed to release lock due to error ${error.message}`);
        }
    }
    async isAlreadyLocked(lockName) {
        let lock;
        try {
            lock = await this.findByName(lockName);
        }
        catch (error) {
            this.logger.debug(`Failed to retrieve existing lock due to error ${error.message}`);
            return false;
        }
        if (!lock) {
            return false;
        }
        const lockTimeout = new Date(lock.timeoutAt).getTime();
        if (Date.now() < lockTimeout) {
            return true;
        }
        await this.delete(lock.id);
        return false;
    }
    async findByName(name) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({}).where('name', name);
        return this.lockDao.findOne(queryBuilder.build());
    }
    async delete(lockID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({}).where('id', lockID);
        const deleteResponse = await this.lockDao.delete(queryBuilder.build());
        return deleteResponse;
    }
    setLockTimeoutAtInterval(lockID, lockTimeout) {
        const updateTimeout = setTimeout(async () => {
            try {
                const lock = await this.updateLockTimeout(lockID, lockTimeout);
                if (lock == null) {
                    clearTimeout(updateTimeout);
                    return;
                }
                this.setLockTimeoutAtInterval(lockID, lockTimeout);
            }
            catch (error) {
                clearTimeout(updateTimeout);
                this.logger.warn(`Failed updated lock timeout due to error ${error.message} `);
            }
        }, exports.LockTimerIntervalInMs);
    }
    async updateLockTimeout(id, lockTimeoutInMs) {
        const lock = await this.lockDao.findById(id);
        if (lock != null) {
            const timeoutAt = new Date(Date.now() + lockTimeoutInMs);
            return this.lockDao.updateByInstance(lock, { timeoutAt: timeoutAt });
        }
        return lock;
    }
}
exports.LockService = LockService;
//# sourceMappingURL=LockService.js.map